/*
    8-5-09
    Copyright Spark Fun Electronics 2007
    Viliam Klein
	Viliam at Sparkfun.com
    
    JPEG trigger board test code
*/

#include <stdio.h>
#include <avr/io.h>
#include <avr/interrupt.h>

#define MYUBRR 15//(((((FOSC * 10) / (16L * BAUD)) + 5) / 10) - 1)

#define sbi(var, mask)   ((var) |= (uint8_t)(1 << mask))
#define cbi(var, mask)   ((var) &= (uint8_t)~(1 << mask))

//==================//
//General Defines
//==================//
#define LED 1//PORTB
#define EEaddress 0x0100//EEPROM address for ADC compare values

//Define functions
//======================
void ioinit(void);      // initializes IO
static int uart_putchar(char c, FILE *stream);
uint8_t uart_getchar(void);

static FILE mystdout = FDEV_SETUP_STREAM(uart_putchar, NULL, _FDEV_SETUP_WRITE);

void delay_ms(uint16_t x); // general purpose delay
void blink(void);

void send_cmd(uint8_t cmd[]);
void flush(void);
uint8_t check_recv(uint8_t check[], uint8_t offset);



//===========//
//Global Vars//
//===========//
uint8_t sync[6] = {0xAA, 0x0D, 0x00, 0x00, 0x00, 0x00};
uint8_t ack[6] = {0xAA, 0x0E, 0x00, 0x00, 0x00, 0x00};
uint8_t acksync[6] = {0xAA, 0x0E, 0x0D, 0x00, 0x00, 0x00};

volatile uint8_t in[256];
volatile uint8_t pointer = 0;
volatile uint8_t status = 0;
//===========//

ISR(USART_RX_vect)
{
	cli();
	cbi(PORTB, LED);
	status = 1;
	
	in[pointer++] = UDR0;
	
	sbi(PORTB, LED);
	sei();
}

int main (void)
{	
	uint8_t f1=0;
	uint8_t f2=0;
	
	ioinit();
	
	flush();
	/*
	printf("\nIN is: ");
	for(int i=0; i<256; i++){printf("%x ",in[i]);}
	*/
	sbi(PORTB, LED);
	delay_ms(5000);	
		
	sei();
	
	while(status!=1)
	{
		send_cmd(sync);
		delay_ms(20);
	}
	
	f1 = check_recv(acksync,12);
	f2 = check_recv(sync,6);
	
	switch(f1)
	{
		case 0:
			break;
		
		case 1:
			blink();
			break;
	}
	
	switch(f2)
	{
		case 0:
			break;
		
		case 1:
			blink();
			break;
	}
	
		
	
	/*sbi(PORTB,LED);
	delay_ms(3000);
	cbi(PORTB,LED);
	delay_ms(3000);
	sbi(PORTB,LED);
	delay_ms(3000);
	
	while((UCSR0A&(1<<UDRE0))==0);
	UDR0 = pointer;
	
	for(int i=0; i<pointer; i++)
	{
		while((UCSR0A&(1<<UDRE0))==0);
		UDR0 = in[i];
	}*/
	
	
	/*while(((UCSR0A&0x80)!=0x80) && (x<60))
	//while(((UCSR0A&0x80)!=0x80))
	{			
		sbi(PORTB,LED);		
		delay_ms(2);
		cbi(PORTB,LED);
		delay_ms(2);
		
		send_cmd(sync);
		x++;		
		
		sbi(PORTB,LED);		
		delay_ms(2);
		cbi(PORTB,LED);
		delay_ms(2);
	}
	
	x=0;
	for(int i=0; i<9; i++)
	{
		sbi(PORTB,LED);
		while((UCSR0A&0x80)!=0x80);
		cbi(PORTB,LED);
		in[x] = UDR0;
		x++;
	}
	
	//sbi(PORTB,LED);
	//sbi(UCSR0B,RXEN0);
	
	delay_ms(8);
	send_cmd(sync);
	delay_ms(8);
	send_cmd(sync);
	
	for(int i=0; i<6; i++)
	{
		sbi(PORTB,LED);
		while((UCSR0A&0x80)!=0x80);
		cbi(PORTB,LED);
		in[x] = UDR0;
		x++;
	}
	
	
	
	sbi(PORTB,LED);
	delay_ms(3000);
	cbi(PORTB,LED);
	delay_ms(3000);
	sbi(PORTB,LED);
	delay_ms(3000);
	
	//printf("\nin[]=\t");
	while((UCSR0A&(1<<UDRE0))==0);
	UDR0 = x;
	
	for(int i=0; i<16; i++)
	{
		while((UCSR0A&(1<<UDRE0))==0);
		UDR0 = in[i];
	}*/
	
	return(0);
}

uint8_t check_recv(uint8_t check[], uint8_t offset)
{
	uint8_t temp[6];
	uint8_t x=0, stat=0;
	
	for(int i=(pointer-offset); i<(pointer-offset+6); i++)
	{
		temp[x] = in[i];
		x++;
	}
	
	if(temp[0]==0xAA){stat=0;}
	else{return 1;}
	
	if(temp[1]==check[1]){stat=0;}
	else{return 1;}
	
	if(temp[2]==check[2]){stat=0;}
	else{return 1;}
	
	/*for(int i=0; i<6; i++)
	{
		while((UCSR0A&(1<<UDRE0))==0);
		UDR0 = temp[i];
	}*/
	
	return stat;
}

void flush(void)
{
	for(int i=0; i<256; i++)
	{
		in[i] = 0xff;
	}
}

void send_cmd(uint8_t cmd[])
{
	for(int i=0; i<6; i++)
	{
		while((UCSR0A&(1<<UDRE0))==0);
		//loop_until_bit_is_set(UCSR0A, UDRE0);
		UDR0 = cmd[i];
	}
	
}

void blink(void)
{
	while(1)
	{
		sbi(PORTB, LED);
		delay_ms(50);
		cbi(PORTB, LED);
		delay_ms(50);
	}
}
	

void ioinit (void)
{
    //1 = output, 0 = input
    DDRB = 0b11101110; //MISO on PB4, CD on PB0
    DDRC = 0b11111000; //
    DDRD = 0b11110010; //RX on PD0, INT0 and INT1 on 2 and 3

    //USART Baud rate: 9600
    UBRR0H = (MYUBRR >> 8);
    UBRR0L = MYUBRR;
	UCSR0C = (1 << UCSZ01) | (1 << UCSZ00);
    UCSR0B = (1<<RXEN0)|(1<<TXEN0)|(1<<RXCIE0);
	UCSR0A = (1<<U2X0);
	
    
    stdout = &mystdout; //Required for printf init
}

static int uart_putchar(char c, FILE *stream)
{
    if (c == '\n') uart_putchar('\r', stream);
  
    loop_until_bit_is_set(UCSR0A, UDRE0);
    UDR0 = c;
    
    return 0;
}

uint8_t uart_getchar(void)
{
    while( !(UCSR0A & (1<<RXC0)) );
    return(UDR0);
}

//General short delays
void delay_ms(uint16_t x)
{
  uint8_t y, z;
  for ( ; x > 0 ; x--){
    for ( y = 0 ; y < 50 ; y++){
      for ( z = 0 ; z < 50 ; z++){
        asm volatile ("nop");
      }
    }
  }
}